module Qdsl
  class Call
    attr_reader :source, :name

    def initialize(source, function_name, parameters, name = nil)
      @source = source
      @function_name = function_name
      @parameters = parameters
      @name = name ? name.to_s : nil
    end

    def as(name)
      Call.new(@source, @function_name, @parameters, name)
    end

    def exists?
      @parameters.all? { |x| x.exists? }
    end

    def render_sql(context, id)
      parameter_results = @parameters.collect { |x| x.render_sql(context, id) }
      query_parameters = parameter_results.collect(&:parameters).inject({}) { |acc, x| acc.merge!(x) }
      call = "#{@function_name}(#{parameter_results.collect(&:sql).join(', ')})"
      SimpleRenderResult.new(@name ? "#{call} AS #{@name}" : call, query_parameters)
    end
  end
end

