module Qdsl
  class Column
    attr_reader :name, :alias_name, :source

    def self.[](obj)
      if obj.is_a?(Column)
        obj
      elsif obj.is_a?(String) || obj.is_a?(Symbol)
        Column.new(obj)
      else
        raise "Cannot convert object of class \"#{obj.class.name}\" to column"
      end
    end

    def initialize(name, alias_name = nil, source = nil)
      @name = name.to_s
      @alias_name = alias_name.nil? ? nil : alias_name.to_s
      @source = source
    end

    def with_name(name)
      Column.new(name, @alias_name, @source)
    end

    def with_alias_name(alias_name)
      Column.new(@name, alias_name, @source)
    end
    alias_method :as, :with_alias_name

    def with_source(source)
      Column.new(@name, @alias_name, source)
    end

    def exists?
      @source.column_names.include?(@name)
    end

    def to_expression
      true?
    end

    def true?
      IsTrue.new(self)
    end

    def render_sql(context, ids)
      SimpleRenderResult.new(@alias_name ? "#{@source.id}.#{@name} AS #{@alias_name}" : "#{@source.id}.#{@name}", {})
    end

    def equals(column)
      Equals.new(self, column)
    end
  end
end

