module Qdsl
  class ColumnSet
    attr_reader :columns, :block

    def initialize(columns, block)
      @columns = columns.collect { |x| Column[x] }
      @block = block
    end

    def capture(source, join_sources)
      sources = [source] + join_sources
      columns = @columns.collect { |x| x.with_source(source) } + if @block
        proxies = sources.collect { |x| ColumnProxy.new(x) }
        [*@block.call(Util::fix_block_params(proxies))]
      else
        []
      end

      undefined_columns = columns.select { |x| !x.exists? }
      raise "One or more undefined columns: #{undefined_columns.collect(&:name).join(', ')}" unless undefined_columns.empty?
      columns
    end
  end
end

