module Qdsl
  class FormatCall
    attr_reader :source, :name

    def initialize(source, format, parameters, name = nil)
      @source = source
      @format = format
      @parameters = parameters
      @name = name.to_s
    end

    def as(name)
      FormatCall.new(@source, @format, @parameters, name)
    end

    def exists?
      @parameters.all? { |x| x.exists? }
    end

    def render_sql(context, id)
      parameter_results = @parameters.collect { |x| x.render_sql(context, id) }
      query_parameters = parameter_results.collect(&:parameters).inject({}) { |acc, x| acc.merge!(x) }
      call = do_format(@format, parameter_results.collect(&:sql))
      SimpleRenderResult.new(@name ? "#{call} AS #{@name}" : call, query_parameters)
    end

    private

    def do_format(format, parameters)
      format.gsub /%(\d+)/ do |m, x|
        index = Integer(Regexp.last_match[1]) - 1
        parameters[index]
      end
    end
  end
end

